# Nimble Gateway Url to make requests to

# Response Messages
SUCCESSFUL = "Operation successful"
INTERNAL_SERVER_ERROR = "An error has occured, please try again later"
INVALID_INPUT = "You have not specified all the inputs or the format of some of the input is wrong"
INVALID_AUTHORIZATION = "You do not have the permission to make this call"

# List of codes that Nimble Helper supports
CODE_LIST = [200, 201, 400, 401, 403, 500]
SUCCESS_CODES = [200, 201]




from rest_framework.viewsets import GenericViewSet
from rest_framework.serializers import Serializer
from rest_framework.response import Response
from nimblehelper.helper import NimbleHelper
from .utils import Utils
from .decorators import dynamic_log_level


class BaseView(GenericViewSet):
    @staticmethod
    def __placeholder_function(x_consumer_id, params, nim_headers=None):
        if not nim_headers:
            nim_headers = dict()
        return {'status': 500, 'x_consumer_id': x_consumer_id, 'params': params, 'nim_headers': nim_headers}

    serializer_class = Serializer

    list_api_function = __placeholder_function
    list_fields = []
    list_required_fields = []

    create_api_function = __placeholder_function
    create_fields = []
    create_required_fields = []

    retrieve_api_function = __placeholder_function
    retrieve_fields = []
    retrieve_required_fields = []

    update_api_function = __placeholder_function
    update_fields = []
    update_required_fields = []

    @classmethod
    @dynamic_log_level()
    def list(cls, request):
        nim_headers = Utils.get_nim_headers(request)
        params = NimbleHelper.check_get_parameters(request=request, fields=cls.list_fields,
                                                   required_fields=cls.list_required_fields)
        response = cls.list_api_function(x_consumer_id=params["x_consumer_id"],
                                         params=params["data"], nim_headers=nim_headers)
        return Response(response, status=response["status"], headers=nim_headers)

    @classmethod
    @dynamic_log_level()
    def create(cls, request):
        nim_headers = Utils.get_nim_headers(request)
        params = NimbleHelper.check_post_parameters(request=request, fields=cls.create_fields,
                                                    required_fields=cls.create_required_fields)
        response = cls.create_api_function(x_consumer_id=params["x_consumer_id"],
                                           params=params["data"], nim_headers=nim_headers)
        return Response(response, status=response["status"], headers=nim_headers)

    @classmethod
    @dynamic_log_level()
    def retrieve(cls, request, pk):
        nim_headers = Utils.get_nim_headers(request)
        params = NimbleHelper.check_get_parameters(request=request, fields=cls.retrieve_fields,
                                                   required_fields=cls.retrieve_required_fields, pk=pk)
        response = cls.retrieve_api_function(x_consumer_id=params['x_consumer_id'], params=params['data'], nim_headers=nim_headers)
        return Response(response, status=response["status"], headers=nim_headers)

    @classmethod
    @dynamic_log_level()
    def update(cls, request, pk):
        nim_headers = Utils.get_nim_headers(request)
        params = NimbleHelper.check_put_parameters(request=request, fields=cls.update_fields,
                                                   required_fields=cls.update_required_fields, pk=pk)
        response = cls.update_api_function(x_consumer_id=params['x_consumer_id'], params=params['data'], nim_headers=nim_headers)
        return Response(response, status=response["status"], headers=nim_headers)



from rest_framework.views import exception_handler
from rest_framework.exceptions import APIException
from .settings import INVALID_INPUT, INVALID_AUTHORIZATION, INTERNAL_SERVER_ERROR


def custom_exception_handler(exc, context):
    # Call REST framework's default exception handler first,
    # to get the standard error response.
    response = exception_handler(exc, context)
    # Now add the HTTP status code to the response.
    if response is not None:
        response.data['status'] = response.status_code
    return response


class BadInputException(APIException):
    status_code = 400
    default_detail = INVALID_INPUT


class InvalidAuthentication(APIException):
    status_code = 401
    default_detail = INVALID_AUTHORIZATION


class InternalServerError(APIException):
    status_code = 500
    default_detail = INTERNAL_SERVER_ERROR



import re
import math


class Utils:
    @classmethod
    def get_nim_headers(cls, request):
        NIM_HEADER_REG_EXP = r"^HTTP\_NIM\_"
        HTTP_REG_EXP = r"^HTTP\_"
        nim_headers = {}
        for key,value in request.META.items():
            if re.match(NIM_HEADER_REG_EXP, key):
                transformed_key = re.sub(HTTP_REG_EXP, "", key).replace("_", "-")
                nim_headers[transformed_key] = value
        return nim_headers

    @classmethod
    def adjust_string_length(cls, value, max_length):
        TRUNCATE_INDICATOR = "---[TRUNCATED]---"
        output = value
        if max_length and max_length.upper() != "OFF":
            max_length = int(max_length)
            value = value.decode("UTF-8")
            length = len(value)
            if length > max_length:
                extra_chars = int(math.floor((length - max_length) / 2))
                middle = int(math.floor(length / 2))
                output = "".join([value[:middle - extra_chars], TRUNCATE_INDICATOR, value[middle + extra_chars:]])
        return output



import logging
from functools import wraps
from django.conf import settings


def process_dynamic_log_level(request):
    logger = logging.getLogger("django.request")
    urllib3_logger = logging.getLogger("urllib3")
    if settings.NIM_DJANGO_REQUEST_LOG_LEVEL_NAME in request.META:
        level_value = request.META.get(settings.NIM_DJANGO_REQUEST_LOG_LEVEL_NAME).upper()
        if level_value in settings.ALLOWED_DYNAMIC_DEBUG_LEVEL_VALUES:
            logger.setLevel(level_value)
            urllib3_logger.setLevel(level_value)
            logger.info("The header '{}' has been overwritten to the value {}".format(
                settings.NIM_DJANGO_REQUEST_LOG_LEVEL_NAME, level_value))
        else:
            logger.info("The header '{}' has an invalid value, the values allowed are {}".format(
                settings.NIM_DJANGO_REQUEST_LOG_LEVEL_NAME, ", ".join(settings.ALLOWED_DYNAMIC_DEBUG_LEVEL_VALUES)))


def dynamic_log_level():
    def _dynamic_log_level(func):  # pragma: no cover
        def dynamic_log_level_logic(*argv, **kwargs):
            if "request" in kwargs:
                request = kwargs["request"]
                process_dynamic_log_level(request)
            elif len(argv) > 1: #The first argument is the Django Rest HTTP request
                request = argv[1]
                process_dynamic_log_level(request)
            return func(*argv, **kwargs)

        return wraps(func)(dynamic_log_level_logic)

    return _dynamic_log_level



from setuptools import setup

setup(name='nimblehelper',
      version='0.6.32',
      description='Nimble Infrastructure Helper',
      author='Brendan Kamp',
      author_email='brendan@tangentsolutions.co.za',
      license='MIT',
      packages=['nimblehelper'],
      install_requires=[
          'djangorestframework==3.3.2',
          'requests',
          'django==1.9.2'
      ],
      zip_safe=False)
